function [x, resBiCGstab] = bicgstab(A,b,x0,maxiter,tol,pmethod);

%function [x,errv] = bicgstabr(A,b,x,maxiter,tol,pmethod)
% function [x,errv] = bicgstabr(K,z,x0,maxiter,tol,pmethod)
% BiCGSTAB by H. Van Der Vorst, by M. Grote, summer 92
% M is an approximate right inverse of A
% preconditioned version:
% calls external functions: matvec and Msolve.
% M here is a dummy variable not the preconditioner. The preconditioner is
% defined globally in Msolve.

M=speye(size(A));

x = x0;

%[n,n] = size(A);
n = size(b,1);
r = b-matvec(A,x);
Norm_r=norm(r);
r0_norm = Norm_r;
resBiCGstab(1)=1;
v = zeros(n,1);
p = v;
oldrho = 1;
alpha = 1;
oldomega = 1;
rhat = r;
k = 0;


while ( k<maxiter & resBiCGstab(k+1)>tol )

   k = k+1;
   rho = rhat'*r;
   beta = (rho/oldrho)*(alpha/oldomega);
   p = r + beta*(p-oldomega*v);
   v = matvec(A,p);

   alpha = rho/(rhat'*v);
   r = r - alpha*v;
   t = matvec(A,r);

   omega = (t'*r)/(t'*t);
   x = x + alpha*p + omega*r;
   r = b-matvec(A,x);

   oldrho = rho;
   oldomega = omega;
   Norm_r=norm(r);
   resBiCGstab(k+1)=Norm_r/r0_norm;
end
nbiter = k;

return


